#!/usr/bin/python3

"""List devices in a rack"""

import requests
import nb2an.netbox
import collections

try:
    from rich import print
except Exception:
    pass

from argparse import ArgumentParser, ArgumentDefaultsHelpFormatter, FileType
from logging import debug, info, warning, error, critical
import logging
import sys


def parse_args():
    "Parse the command line arguments."
    parser = ArgumentParser(
        formatter_class=ArgumentDefaultsHelpFormatter,
        description=__doc__,
        epilog="Exmaple Usage: ",
    )

    parser.add_argument(
        "--log-level",
        default="info",
        help="Define the logging verbosity level (debug, info, warning, error, fotal, critical).",
    )

    parser.add_argument("rack", type=str, help="Rack choice")

    args = parser.parse_args()
    log_level = args.log_level.upper()
    logging.basicConfig(level=log_level, format="%(levelname)-10s:\t%(message)s")
    return args


def main():
    args = parse_args()

    by_outlet = {}
    by_device = collections.defaultdict(list)

    nb = nb2an.netbox.Netbox()
    r = nb.get("/dcim/devices/?rack_id=" + args.rack)
    for device in r:
        outlets = nb.get_outlets_by_device_id(device["id"])
        print(f"{device['display']}:")
        for outlet in outlets:
            print(f"  - {outlet['device']['display']}.{outlet['display']}")
            by_outlet[outlet["display"]] = outlet["device"]["display"]
            by_device[outlet["device"]["display"]].append(outlet["display"])

    # print(by_outlet)
    # print(by_device)


if __name__ == "__main__":
    main()



#!/usr/bin/python3

import requests
import nb2an.netbox

try:
    from rich import print
except Exception:
    pass


def main():
    # args = parse_args()

    racks = nb2an.netbox.Netbox().get_racks()
    print(f"{'Id':<3} {'Name':<25} {'Site':<20} {'Location':<20} {'#devs'}")
    for rack in racks:
        print(
            f"{rack['id']:<3} {rack['display']:<25} {rack['site']['display']:<20} {rack['location']['display']:<20} {rack['device_count']}"
        )


if __name__ == "__main__":
    main()



#!/usr/bin/python3

class DotNest():
    def __init__(self, data):
        self._data = data

    @property
    def data(self):
        return self._data

    @data.setter
    def data(self, newdata):
        self._data = newdata

    def get(self, keys):
        """given a list of keys, return the value at spot

           keys must be a list/tuple of dict keys or ints for list elements"""
        keys = self.parse_keys(keys)
        ptr = self.data

        for n, k in enumerate(keys):
            if isinstance(ptr, dict) and k not in ptr:
                raise ValueError(f"key #{n} '{k}' not found in data")
            if isinstance(ptr, list):
                if isinstance(k, str):
                    k = int(k)
                if len(ptr) <= k:
                    raise ValueError(f"list key #{n} int({k}) too large")
            ptr = ptr[k]

        return ptr

    def set(self, keys, value):
        "given a list of keys, set the value at that spot to a new value"
        keys = self.parse_keys(keys)
        ptr = self.get(keys[0:-1])
        ptr[keys[-1]] = value

    def parse_keys(self, values):
        if isinstance(values, list):
            return values
        # TODO: allow / pathing if values starts with a /?
        # TODO: deal with escapes
        return values.split(".")

    def __eq__(self, other):
        return self.deep_compare(self.data, other.data)

    # from https://stackoverflow.com/questions/25044073/comparing-python-objects-from-deepcopy
    def deep_compare(self, left, right, excluded_keys = []):
        # convert left and right to dicts if possible, skip if they can't be converted
        try: 
            left = left.__dict__
            right = right.__dict__
        except:
            pass

        # both sides must be of the same type 
        if type(left) != type(right):
            return False

        # compare the two objects or dicts key by key
        if type(left) == dict:
            for key in left:
                # make sure that we did not exclude this key
                if key not in excluded_keys:
                    # check if the key is present in the right dict, if not, we are not equals
                    if key not in right:
                        return False
                    else:
                        # compare the values if the key is present in both sides
                        if not self.deep_compare(left[key], right[key], excluded_keys):
                            return False

            # check if any keys are present in right, but not in left
            for key in right:
                if key not in left and key not in excluded_keys:
                    return False
            
            return True

        # check for each item in lists
        if type(left) == list:
            # right and left must have the same length
            if len(left) != len(right):
                return False

            # compare each item in the list
            for index in range(len(left)):
                if not self.deep_compare(left[index], right[index], excluded_keys):
                    return False

        # do a standard comparison
        return left == right



#!/usr/bin/python3

"""checks ansible configuration"""

from argparse import ArgumentParser, ArgumentDefaultsHelpFormatter, FileType
from logging import debug, info, warning, error, critical
import logging
import sys
import os

from rich import print
from logging import debug

from ansible.inventory.manager import InventoryManager
from ansible.parsing.dataloader import DataLoader
from ansible.vars.manager import VariableManager
from ansible.template import Templar

import nb2an.netbox


def parse_args():
    "Parse the command line arguments."
    parser = ArgumentParser(formatter_class=ArgumentDefaultsHelpFormatter,
                            description=__doc__,
                            epilog="Exmaple Usage: ")

    parser.add_argument("-i", "--inventory-file", default="hosts", type=str,
                        help="The name of the inventory/hosts list file")

    parser.add_argument("-H", "--hosts", default=None, type=str, nargs="*",
                        help="A specific set of hosts to check")

    parser.add_argument("--log-level", "--ll", default="warning",
                        help="Define the logging verbosity level (debug, info, warning, error, fotal, critical).")

    parser.add_argument("-d", "--ansible-directory",
                        type=str, help="The ansible directory to verify")

    args = parser.parse_args()
    log_level = args.log_level.upper()
    logging.basicConfig(level=log_level,
                        format="%(levelname)-10s:\t%(message)s")
    return args


def main():
    args = parse_args()

    nb = nb2an.netbox.Netbox()
    config = nb.get_config()

    ansible_directory = args.ansible_directory
    if not ansible_directory:
        ansible_directory = config.get("ansible_directory")

    if not ansible_directory or not os.path.exists(ansible_directory):
        error("Failed to find expansible directory")
        exit(1)

    hosts = os.path.join(ansible_directory, args.inventory_file)
    if not os.path.exists(hosts):
        error("Failed to find the path to the inventory file")
        exit(1)

    # setup ansible
    dl = DataLoader()
    im = InventoryManager(loader=dl, sources=[hosts])
    vm = VariableManager(loader=dl, inventory=im)

    check_interfaces(vm, dl, im, nb, args.hosts)


def check_interfaces(vm, dl, im, nb, hosts=None):
    # set up ansible config grabber

    # get netbox devices
    netbox_addrs = nb.get_addresses()

    host_list = netbox_addrs.keys()
    if hosts:
        host_list = hosts
    for host in host_list:

        try:
            # get ansible config for this host
            search_host = host
            if not host.endswith(nb.suffix):
                search_host = search_host + nb.suffix
            ansible_host = im.get_host(search_host)
            vars = vm.get_vars(host=ansible_host)
            templar = Templar(loader=dl, variables=vars)

            if 'net_config' not in vars:
                warning(f"{search_host} does not have \"net_config\" in ansible")
                continue

            netconf = templar.template(vars['net_config'])

            # compare each interface on the host
            if len(netbox_addrs[host]) == 0:
                warning(f"{host}: no interfaces in netbox")
                continue

            for interface in netbox_addrs[host]:
                debug(f"starting {host} {interface}")

                for i in netconf:
                    if 'ipaddr' in i and i['name'] == interface:
                        debug(i)
                        debug(netbox_addrs[host][interface])

                        label = host + "/" + interface + ":"
                        # check v4
                        if 'IPv4' in netbox_addrs[host][interface]:
                            netbox_addr = netbox_addrs[host][interface]['IPv4']
                            netbox_addr = netbox_addr[0:netbox_addr.find("/")]
                            if netbox_addr != i['ipaddr']:
                                warning(f"{label:<25} netbox={netbox_addr:<25} ansible={i['ipaddr']:<25}")
                            else:
                                info(f"{label:<25} IPv4 values match")

                        # check v6
                        if 'IPv6' in netbox_addrs[host][interface]:
                            netbox_addr = netbox_addrs[host][interface]['IPv6']
                            if i['ipv6addr'].rfind("/") == -1:
                                netbox_addr = netbox_addr[0:netbox_addr.find("/")]
                            if netbox_addr != i['ipv6addr']:
                                warning(f"{label:<25} netbox={netbox_addr:<25} ansible={i['ipv6addr']:<25}")
                            else:
                                info(f"{label:<25} IPv6 values match")

                        # stop on match
                        break
                else:
                    debug(f"  no {interface} found in netbox")

        except Exception as exp:
            warning(f"failed with stack trace for host {host}")
            warning("  " + str(exp))


if __name__ == "__main__":
    main()






        if make_changes:
            # # now do modifications
            if 'host_info' not in yaml_struct:
                yaml_struct['host_info'] = {}
            # yaml_struct['host_info']['netbox_modified'] = True

            # if 'net_config' in yaml_struct:
            #     yaml_struct['net_config'].append({
            #         'name': 'nb_dummy0',
            #         'ipaddr': "127.0.0.9",
            #         'prefix': 24})

            #
            # OUTLETS
            #
            debug(f"  checking outlets for {hostname}")
            for o in outlets:
                debug(f"    checking outlet connected to {b.fqdn(o['device']['name'])}")
                if b.fqdn(o['device']['name']) == hostname:
                    debug(f"  FOUND outlet connected to {b.fqdn(o['device']['name'])}")
                    if 'outlets' not in yaml_struct['host_info']:
                        yaml_struct['host_info']['outlets'] = {}
                    yaml_struct['host_info']['outlets'][o['name']] = {
                        'label': o['label'],
                    }

            #
            # DEVICE information
            #
            d = b.get_devices_by_name(hostname)
            if len(d) > 1:
                warning(f"too many devices returned for {hostname}")
            if d:
                d = d[0]
                if 'device_type' in d and \
                   'manufacturer' in d['device_type']:
                    yaml_struct['host_info']['systype'] = d['device_type']['manufacturer']['name']
                    yaml_struct['host_info']['model'] = d['device_type']['model']

                if 'serial'in d:
                    yaml_struct['host_info']['isitag'] = d['serial'].lower()

                # netbox reference
                yaml_struct['netbox'] = {
                    'url': b.prefix + "/dcim/device/" + str(d['id']),
                    'device_id': d['id'],
                    'site': d['site']['name'],
                    'site_url': d['site']['url'],
                    'location': d['location']['name'],
                    'location_url': d['location']['url'],
                    'rack': d['rack']['name'],
                    'rack_url': d['location']['url'],
                }  # always reset everything

            # if we inserted nothing, drop it
            if yaml_struct['host_info'] == {}:
                del yaml_struct['host_info']




#!/usr/bin/python3

"""Reports specific parameters from each device."""

from argparse import ArgumentParser, ArgumentDefaultsHelpFormatter, FileType
from logging import debug, info, warning, error, critical
import logging
import sys
import os
import re
import yaml
import shutil
import subprocess
import ruamel.yaml

import nb2an.netbox
import nb2an.dotnest
from nb2an.plugins.update_ansible import update_ansible_plugins

def parse_args():
    parser = ArgumentParser(
        formatter_class=ArgumentDefaultsHelpFormatter,
        description=__doc__,
        epilog="Exmaple Usage: nb-parameters display device.manufacturer.display",
    )

    parser.add_argument(
        "--log-level",
        "--ll",
        default="info",
        help="Define the logging verbosity level (debug, info, warning, error, fotal, critical).",
    )

    parser.add_argument(
        "-r", "--racks", default=[], type=int, nargs="*", help="Racks to show"
    )

    parser.add_argument("-f", "--fsdb", action="store_true",
                        help="Output data in FSDB format")

    parser.add_argument(
        "-D",
        "--devices",
        default=[],
        type=str,
        nargs="*",
        help="Process only these NetBox device names",
    )

    parser.add_argument("data_specifications", type=str, nargs="*",
                        help="Data specifications to use in nb2an keying format")

    args = parser.parse_args()
    log_level = args.log_level.upper()
    logging.basicConfig(level=log_level, format="%(levelname)-10s:\t%(message)s")
    return args

def process_devices(nb, racks=[], specifications=[], as_fsdb=False):
    devices = nb.get_devices(racks, link_other_information=True)

    if as_fsdb:
        import pyfsdb
        fh = pyfsdb.Fsdb(out_file_handle=sys.stdout)
        fh.out_column_names = ['name'] + [x for x in specifications]

    for device in devices:
        name = nb.fqdn(device["name"])
        dn = nb2an.dotnest.DotNest(device)

        # FSDB output
        if as_fsdb:
            row = [name]
            for specification in specifications:
                try:
                    row.append(dn.get(specification))
                except Exception:
                    row.append(None)
            fh.append(row)
            continue

        # human output
        print(f"{name}")
        for specification in specifications:
            try:
                print(f"  {specification:<40s}:  {dn.get(specification)}")
            except Exception:
                print(f"  {specification:<40s}:  [DNE]")

def main():
    args = parse_args()
    nb = nb2an.netbox.Netbox()
    config = nb.get_config()

    process_devices(nb, racks=args.racks, specifications=args.data_specifications, as_fsdb=args.fsdb)



if __name__ == "__main__":
    main()



from logging import debug, info, warning, error, critical
import re
update_ansible_plugins = {}

from functools import wraps


def plugin(function):
    # register it
    fn_name = function.__name__
    fn_name = fn_name.replace("fn_", "", 1)
    update_ansible_plugins[fn_name] = function

    @wraps(function)
    def _wrap(*args, **kwargs):
        return function(*args, **kwargs)

    return _wrap


def keys_present(definition: dict, keys: list[str]):
    for key in keys:
        if key not in definition:
            error(f"Failed to find key '{key}' in {definition}")
            return False
    return True


@plugin
def fn_replace(dn, yaml_struct, definition, item):
    "A function to do internal string replacements before setting a value"
    if not keys_present(definition, ['value', 'search', 'replacement']):
        return
    value = dn.get(definition['value'])
    search = definition['search']
    replacement = definition['replacement']
    newvalue = re.sub(search, replacement, value,
                      count=definition.get('count', 0))
    yaml_struct[item] = newvalue


@plugin
def fn_delete(dn, yaml_struct, definition, item):
    "A plugin to remove a node"
    if item in yaml_struct:
        del yaml_struct[item]


@plugin
def fn_foreach_create_dict(dn, yaml_struct, definition, item):
    "iterates over a definition and applies it multiple times per array, replacing the contents of the original structure"
    # clear the existing content
    yaml_struct[item] = {}

    fn_foreach_augment_dict(dn, yaml_struct, definition, item)


@plugin
def fn_foreach_augment_dict(dn, yaml_struct, definition, item):
    "iterates over a definition and applies it multiple times per array, augmenting an existing structure."
    if not keys_present(definition, ['structure', 'array', 'keyname']):
        return

    # clear the existing content
    yaml_struct[item] = {}

    # for each item from the netbox array, create a structure
    array_name = definition['array']
    array = dn.get(definition['array'])
    starting_structure = definition['structure']
    for n, substructure in enumerate(array):
        replacement = {}
        keyvalue = dn.get(f"{array_name}.{n}.{definition['keyname']}")
        for subitem in starting_structure:
            path = f"{array_name}.{n}.{starting_structure[subitem]}"
            value = dn.get(path)
            replacement[subitem] = value
        yaml_struct[item][keyvalue] = replacement




#!/usr/bin/python3

"""List networks in a rack"""

import requests
import nb2an.netbox
import collections

try:
    from rich import print
except Exception:
    pass

from argparse import ArgumentParser, ArgumentDefaultsHelpFormatter, FileType
from logging import debug, info, warning, error, critical
import logging
import sys


def parse_args():
    "Parse the command line arguments."
    parser = ArgumentParser(
        formatter_class=ArgumentDefaultsHelpFormatter,
        description=__doc__,
        epilog="Exmaple Usage: ",
    )

    parser.add_argument(
        "--log-level",
        "--ll",
        default="info",
        help="Define the logging verbosity level (debug, info, warning, error, fotal, critical).",
    )

    parser.add_argument("rack", type=str, help="Rack choice")

    args = parser.parse_args()
    log_level = args.log_level.upper()
    logging.basicConfig(level=log_level, format="%(levelname)-10s:\t%(message)s")
    return args


def main():
    args = parse_args()

    nb = nb2an.netbox.Netbox()
    r = nb.get("/dcim/devices/?rack_id=" + args.rack)

    interface_addresses = nb.get_addresses()
    interfaces = nb.get_interfaces()

    for device in sorted(r, key=lambda x: x["display"]):

        name = device["display"]
        print(f"{name}:")

        # get interfaces
        dev_interfaces = interfaces[name]

        for interface in dev_interfaces:
            interface = dev_interfaces[interface]
            other_end = ""
            if (
                "cable_peer" in interface
                and interface["cable_peer"]
                and "device" in interface["cable_peer"]
            ):
                other_end = interface["cable_peer"]["device"]["display"]

            ifname = ""
            if "display" in interface:
                ifname = interface["display"]

            ipv4 = ""
            if ifname in interface_addresses[name]:
                ipv4 = interface_addresses[name][ifname].get("IPv4", "")

            ipv6 = ""
            if ifname in interface_addresses[name]:
                ipv6 = interface_addresses[name][ifname].get("IPv6", "")

            print(
                f"  {interface['name']:<12} {other_end:<10} {ipv4:<20} {ipv6:<30} {interface['type']['label']}"
            )


if __name__ == "__main__":
    main()



#!/usr/bin/python3

"""Updates ansible YAML files with information from netbox"""

from argparse import ArgumentParser, ArgumentDefaultsHelpFormatter, FileType
from logging import debug, info, warning, error, critical
import logging
import sys
import os
import re
import yaml
import shutil
import subprocess
import ruamel.yaml

import nb2an.netbox
import nb2an.dotnest
from nb2an.plugins.update_ansible import update_ansible_plugins

PLUGIN_KEY = "__function"


def parse_args():
    parser = ArgumentParser(
        formatter_class=ArgumentDefaultsHelpFormatter,
        description=__doc__,
        epilog="Exmaple Usage: update",
    )

    parser.add_argument(
        "-n", "--noop", action="store_true", help="Don't actually make changes"
    )

    parser.add_argument(
        "--log-level",
        "--ll",
        default="info",
        help="Define the logging verbosity level (debug, info, warning, error, fotal, critical).",
    )

    parser.add_argument(
        "-r", "--racks", default=[], type=int, nargs="*", help="Racks to update"
    )

    parser.add_argument(
        "-d", "--ansible-directory", type=str, help="The ansible directory to verify"
    )

    parser.add_argument(
        "-D",
        "--devices",
        default=[],
        type=str,
        nargs="*",
        help="Process only these NetBox device names",
    )

    parser.add_argument(
        "-c",
        "--changes-file",
        default=None,
        type=FileType("r"),
        help="The changes definition file to use",
    )

    parser.add_argument(
        "-w",
        "--whitespace-hack",
        action="store_true",
        help="Try a multi-run with yaml-edit and whitespace ignoring diff to patch minimally",
    )

    args = parser.parse_args()
    log_level = args.log_level.upper()
    logging.basicConfig(level=log_level, format="%(levelname)-10s:\t%(message)s")
    return args


def process_changes(changes, yaml_struct, nb_data):
    dn = nb2an.dotnest.DotNest(nb_data)
    for item in changes:
        if isinstance(changes[item], dict):

            # check if it's a special dict with instructions
            if PLUGIN_KEY in changes[item]:
                function_name = changes[item][PLUGIN_KEY]
                if function_name not in update_ansible_plugins:
                    error(f"function '{function_name}' is unknown")
                    exit(1)

                try:
                    fn = update_ansible_plugins[function_name]
                    value = fn(dn, yaml_struct, changes[item], item)
                except Exception:
                    debug(f"failed to call function {function_name}")
            else:
                if item not in yaml_struct:
                    yaml_struct[item] = {}  # TODO: allow list creation
                process_changes(changes[item], yaml_struct[item], nb_data)


            # if nothing was added, drop it again
            if item in yaml_struct and yaml_struct[item] == {}:
                del yaml_struct[item]
        elif isinstance(changes[item], str):
            try:
                value = dn.get(changes[item])
                yaml_struct[item] = value
            except Exception:
                debug(f"skipping {changes[item]}: failed to find netbox value")


def process_host(
    nb: nb2an.netbox.Netbox,
    hostname: str,
    yaml_file: str,
    changes: dict = None,
):
    info(f"modifying {yaml_file}")

    # load the original YAML
    with open(yaml_file) as original:
        yaml_data = original.read()

        yaml_parser = ruamel.yaml.YAML()
        yaml_parser.indent(mapping=2, sequence=4, offset=2)
        yaml_parser.preserve_quotes = True
        yaml_parser.width = 4096
        yaml_struct = yaml_parser.load(yaml_data)

    if changes:
        nb_data = nb.get_devices_by_name(hostname, link_other_information=True)
        if not nb_data or len(nb_data) != 1:
            info(f"not processing changes for {hostname} as no netbox data found")
        else:
            nb_data = nb_data[0]
            process_changes(changes, yaml_struct, nb_data)

        for item in changes:
            debug(f"setting: {item} to {changes[item]}")

    # write the YAML back out
    with open(yaml_file, "w") as modified:
        yaml_parser.dump(yaml_struct, modified)


def process_devices(nb, ansible_directory, racks=[], changes=True):
    devices = nb.get_devices(racks, link_other_information=True)

    for device in devices:
        name = nb.fqdn(device["name"])
        debug(f"starting: {name}")

        device_yaml = os.path.join(ansible_directory, "host_vars", name + ".yml")
        if os.path.exists(device_yaml):
            process_host(nb, name, device_yaml, changes=changes)


def main():
    args = parse_args()
    nb = nb2an.netbox.Netbox()
    config = nb.get_config()

    ansible_directory = args.ansible_directory
    if not ansible_directory:
        ansible_directory = config.get("ansible_directory")
    host_vars = os.path.join(ansible_directory, "host_vars")

    if not ansible_directory:
        error("Failed to find ansible_directory in args or .nb2an config")
        exit(1)

    changes = None
    if not args.noop and args.changes_file:
        changes = yaml.safe_load(args.changes_file.read())

    # maybe copy the info to a separate set of files
    if args.whitespace_hack:
        shutil.copytree(host_vars, host_vars + ".nb2an-bkup")

    process_devices(nb, ansible_directory, racks=args.racks, changes=changes)

    # put the original back
    if args.whitespace_hack:
        # move the original data back
        os.rename(host_vars, host_vars + ".nb2an-modified")
        os.rename(host_vars + ".nb2an-bkup", host_vars)

        # generate a patch
        subprocess.run(["diff", "-wBuZE", host_vars, host_vars + ".nb2an-modified"])


if __name__ == "__main__":
    main()



#!/usr/bin/python3

"""List devices in a rack"""

import requests
import nb2an.netbox

try:
    from rich import print
except Exception:
    pass

from argparse import ArgumentParser, ArgumentDefaultsHelpFormatter, FileType
from logging import debug, info, warning, error, critical
import logging
import sys

def parse_args():
    "Parse the command line arguments."
    parser = ArgumentParser(formatter_class=ArgumentDefaultsHelpFormatter,
                            description=__doc__,
                            epilog="Exmaple Usage: ")

    parser.add_argument("-b", "--blanks", action="store_true",
                        help="Show blank areas with blank lines")

    parser.add_argument("--log-level", default="info",
                        help="Define the logging verbosity level (debug, info, warning, error, fotal, critical).")

    parser.add_argument("rack", type=int, nargs="*", default=None,
                        help="Rack choice (all if not)")

    args = parser.parse_args()
    log_level = args.log_level.upper()
    logging.basicConfig(level=log_level,
                        format="%(levelname)-10s:\t%(message)s")
    return args


def main():
    args = parse_args()

    devices = nb2an.netbox.Netbox().get_devices(args.rack)
    print(f"{'Id':<3} {'Pos':<3} {'Name':<25} {'Type':<20}")
    last_spot = None
    for device in sorted(devices, key=lambda x: x['position'] or 0,
                         reverse=True):
        position = int(device['position'] or 0)
        if last_spot and args.blanks:
            for blank in range(last_spot-1, position, -1):
                print(blank)

        print(f"{device['id']:<3} {position:<3} {device['display']:<25} {device['device_type']['display']:<20}")
        last_spot = position

if __name__ == "__main__":
    main()





import os
import yaml
import requests
import collections
from typing import Union
from logging import debug, error
from rich import print

default_url = "https://netbox/api"
default_config_path = os.path.join(os.environ.get("HOME"), ".nb2an")


class Netbox:
    "An interface to Netbox to extract data needed for nb2an to function"

    def __init__(
        self,
        api_url=default_url,
        config_path=default_config_path,
        ansible_dir=None,
        suffix=None,
    ):
        self.config_path = config_path
        self.config = {}
        if os.path.exists(self.config_path):
            self.config = self.get_config()
        else:
            error(f"you must create a {self.config} configuration file first")
            exit(1)
        self.prefix = self.config.get("api_url", api_url)
        self.suffix = self.config.get("suffix", suffix)
        self.ansible_dir = self.config.get("ansible_dir", ansible_dir)
        self.url_cache = {}
        self.devices_by_id = {}
        self.devices_by_name = {}

        self.data = {}

    def get_config(self):
        debug(f"loading config from {self.config_path}")
        content = open(self.config_path).read()
        # debug(f"{content=}")
        results = yaml.safe_load(str(open(self.config_path).read()))
        return results

    def fqdn(self, hostname):
        if not self.suffix:
            return hostname
        if not hostname.endswith(self.suffix):
            hostname = hostname + self.suffix
        return hostname

    def shortname_name(self, hostname):
        if hostname.endswith(self.suffix):
            return hostname[0 : -len(self.suffix)]
        return hostname

    def get_cached_device_by_name(self, hostname):
        # try whatever they passed
        if hostname in self.devices_by_name:
            return self.devices_by_name[hostname]

        # try shorter
        shorter = self.shortname_name(hostname)
        if shorter in self.devices_by_name:
            return self.devices_by_name[shorter]

        # try longer
        fqdn = self.fqdn(hostname)
        if fqdn in self.devices_by_name:
            return self.devices_by_name[fqdn]

        return None  # whoops

    def get(self, url: str, use_cache: bool = True, strip_results: bool = True):
        "fetch data from a URL, and potentially cache the results"
        if not url.startswith("http"):
            url = self.prefix + url

        if use_cache and url in self.url_cache:
            debug(f"returning cached: {url}")
            return self.url_cache[url]
        debug(f"fetching: {url}")

        c = self.config
        headers = {"Authorization": f"Token {c['token']}"}
        auth = None
        if "user" in c and "password" in c:
            auth = (c["user"], c["password"])

        # debug(f"headers: {headers}")

        # get the contents
        r = requests.get(url, headers=headers, auth=auth)
        r.raise_for_status()

        # maybe cache them
        encoded_results = r.json()
        if strip_results:
            encoded_results = encoded_results["results"]
        if use_cache:
            self.url_cache[url] = encoded_results

        return encoded_results

    def get_racks(self):
        results = self.get("/dcim/racks")
        return results

    def get_devices(
        self,
        racknums: Union[list[int], int] = None,
        link_other_information: bool = False,
    ):
        if isinstance(racknums, int):
            racknums = [racknums]

        devices = []
        if racknums:
            for racknum in racknums:
                rack_devices = self.get("/dcim/devices/?rack_id=" + str(racknum))
                devices.extend(rack_devices)
        else:
            all_devices = self.get("/dcim/devices/")
            devices.extend(all_devices)

        if link_other_information:
            devices = self.link_device_data(devices)
        return devices

    def get_devices_by_id(
        self,
        devices: Union[list[int], int] = None,
        link_other_information: bool = False,
    ):
        "Given a device ID, grab the device's info from the API"
        if isinstance(devices, int):
            devices = [devices]

        # assume we want all devices
        if devices is None or devices == []:
            devices = [x["id"] for x in self.get_devices()]

        # grab each device in turn
        results = []
        for device in devices:
            debug(f"looking for device {device} by id")
            if device in self.devices_by_id:
                results.append(self.devices_by_id[device])
                continue

            the_devices = self.get("/dcim/devices/" + str(device), strip_results=False)
            results.append(the_devices)

        # link in other things
        if link_other_information:
            results = self.link_device_data(results)

        return results

    def get_devices_by_name(
        self,
        devices: Union[list[str], str] = None,
        link_other_information: bool = False,
    ):
        if isinstance(devices, str):
            devices = [devices]

        # assume we want all devices
        if devices is None or devices == []:
            devices = [x["id"] for x in self.get_devices()]

        results = []
        for device in devices:
            debug(f"looking for device {device} by name")
            cached = self.get_cached_device_by_name(device)
            if cached:
                results.append(cached)
                continue
            the_device = self.get("/dcim/devices/?name=" + str(device))
            if (not the_device or len(the_device) != 1) and device.endswith(
                self.suffix
            ):
                # retry without a FQDN suffix
                device = device[0 : -len(self.suffix)]
                the_device = self.get("/dcim/devices/?name=" + str(device))[0]
            elif not the_device:
                # retry WITH a FQDN
                device = self.fqdn(device)
                the_device = self.get("/dcim/devices/?name=" + str(device))[0]
            else:
                the_device = the_device[0]

            if the_device:
                results.append(the_device)

        if link_other_information:
            results = self.link_device_data(results)

        return results

    # generic grabber for things attached to a device
    def get_device_components_by_device_id(self, component: str, device_id: int):
        self.bootstrap_all_data()
        objects = self.data[component]

        results = []
        for obj in objects:
            try:
                if obj["connected_endpoint"]["device"]["id"] == device_id:
                    results.append(obj)

            except Exception:
                pass
        return results

    def get_device_components_by_device_name(self, component: str, device_name: int):
        self.bootstrap_all_data()
        objects = self.data[component]

        results = []
        for obj in objects:
            try:
                if obj["connected_endpoint"]["name"] == device_name:
                    results.append(obj)
            except Exception:
                pass
        return results

    # Outlets
    def get_outlets(self):
        self.bootstrap_all_data()
        return self.data["outlets"]

    @property
    def outlets(self):
        return self.get_outlets()

    def get_outlets_by_device_id(self, device: int):
        return self.get_device_components_by_device_id("outlets", device)

    def get_outlets_by_device_name(self, device: str):
        return self.get_device_components_by_device_name("outlets", device)

    # power ports
    def get_power_ports(self, device: int = None):
        self.bootstrap_all_data()
        return self.data["power_ports"]

    @property
    def power_ports(self):
        return self.get_power_ports()

    def get_power_ports_by_device_id(self, device: int):
        return self.get_device_components_by_device_id("power_ports", device)

    def get_power_ports_by_device_name(self, device: str):
        return self.get_device_components_by_device_name("power_ports", device)

    def get_addresses(self) -> dict:
        "Returns a nested dict of all registered hosts/interface/family = addresses"
        interface_addresses = collections.defaultdict(dict)
        for family in [4, 6]:
            r = self.get(f"/ipam/ip-addresses/?family={family}")
            for addr in r:
                host = addr["assigned_object"]["device"]["display"]
                endpoint = addr["assigned_object"]["name"]
                if endpoint not in interface_addresses[host]:
                    interface_addresses[host][endpoint] = {}
                interface_addresses[host][endpoint][addr["family"]["label"]] = addr[
                    "address"
                ]

        return dict(interface_addresses)

    def get_interfaces(self) -> list:
        r = self.get(f"/dcim/interfaces/")

        interfaces = collections.defaultdict(dict)
        for interface in r:
            device = interface["device"]["name"]
            interfaces[device][interface["display"]] = interface

        return interfaces

    def bootstrap_all_data(self) -> None:
        "pre-fetch all netbox data"
        if "interfaces" not in self.data:
            self.data["interfaces"] = self.get_interfaces()

        if "addresses" not in self.data:
            self.data["addresses"] = self.get_addresses()

        if "devices" not in self.data:
            self.data["devices"] = self.get_devices()

        self.data["outlets"] = self.get("/dcim/power-outlets/")
        self.data["power_ports"] = self.get("/dcim/power-ports/")

        if "power_ports" not in self.data:
            self.data["power_ports"] = self.get_power_ports()

    def link_device_data(self, devices=None) -> dict:
        self.bootstrap_all_data()
        if not devices:
            devices = self.data["devices"]

        for device in devices:
            if device["name"] in self.data["interfaces"]:
                device["interfaces"] = self.data["interfaces"][device["name"]]
            if device["name"] in self.data["addresses"]:
                device["addresses"] = self.data["addresses"][device["name"]]

            device["power_ports"] = []
            for port in self.data["power_ports"]:
                if port["device"]["name"] == device["name"]:
                    device["power_ports"].append(port)

            self.devices_by_id[device["id"]] = device
            self.devices_by_name[device["name"]] = device

        return devices

    def get_interfaces_by_device_name(self, device_name: str) -> dict:
        self.bootstrap_all_data()

        interfaces = self.data["interfaces"][device_name]
        return interfaces



#!/usr/bin/python3

"""List attributes of a device"""

import requests
import nb2an.netbox

try:
    from rich import print
except Exception:
    pass

from argparse import ArgumentParser, ArgumentDefaultsHelpFormatter, FileType
from logging import debug, info, warning, error, critical
import logging
import sys
import yaml


def parse_args():
    "Parse the command line arguments."
    parser = ArgumentParser(
        formatter_class=ArgumentDefaultsHelpFormatter,
        description=__doc__,
        epilog="Exmaple Usage: ",
    )

    parser.add_argument(
        "-b", "--blanks", action="store_true", help="Show blank areas with blank lines"
    )

    parser.add_argument(
        "--log-level",
        "--ll",
        default="info",
        help="Define the logging verbosity level (debug, info, warning, error, fotal, critical).",
    )

    parser.add_argument(
        "devices", type=str, nargs="*", default=None, help="Device number"
    )

    args = parser.parse_args()
    log_level = args.log_level.upper()
    logging.basicConfig(level=log_level, format="%(levelname)-10s:\t%(message)s")
    return args


def main():
    args = parse_args()

    nb = nb2an.netbox.Netbox()

    for device in args.devices:
        try:
            debug(f"trying by id: {device}")
            id = int(device)
            device = nb.get_devices_by_id(id)
        except Exception:
            debug(f"trying by name: {device}")
            device = nb.get_devices_by_name(device)

        device = nb.link_device_data(device)

        for innerdevice in device:  # could have returned more than one
            print(f"#\n# device: #{innerdevice['id']}\n#")
            print(yaml.dump(innerdevice))


if __name__ == "__main__":
    main()



import setuptools

with open("README.md", "r") as fh:
    long_description = fh.read()

setuptools.setup(
    name="nb2an",
    version="0.9.6",
    author="Wes Hardaker",
    author_email="opensource@hardakers.net",
    description="A set of python scripts to compare or update NetBox and Ansible",
    long_description=long_description,
    long_description_content_type="text/markdown",
    url="https://github.com/hardaker/nb2an",
    packages=setuptools.find_packages(),
    entry_points={
        "console_scripts": [
            # migrating to pdb prefixes
            "nb-device = nb2an.tools.getdevice:main",
            "nb-devices = nb2an.tools.getdevices:main",
            "nb-racks = nb2an.tools.getracks:main",
            "nb-outlets = nb2an.tools.getoutlets:main",
            "nb-networks = nb2an.tools.getnetwork:main",
            "nb-update-ansible = nb2an.tools.update_ansible:main",
            "nb-parameters = nb2an.tools.getparemeters:main",
            #            'nb-check-ansible = nb2an.tools.checkansible:main',
        ]
    },
    classifiers=[
        "Programming Language :: Python :: 3",
        "Operating System :: OS Independent",
    ],
    install_requires=[
        "rich",
        "ansible",
        "ruamel.yaml",
        "requests",
        "pyaml",
    ],
    python_requires=">=3.6",
)
